/*************************************************************************************
 * (c) 2003, Roberto A. Foglietta <robang@libero.it>
 *     Released under Licence GPL v2.0
 *     This program is provided "as is" without warranty of any kind.  
 *     Use it at your own risk.
 *************************************************************************************/



#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <gtk/gtk.h>

#include "main.h"
#include "callbacks.h"
#include "interface.h"
#include "support.h"

#define MAX_PASSWORD_LEN 128
char password[MAX_PASSWORD_LEN] = {"\0"};

#define CREATE_KEY_ERROR  "\ndevi riempire entrambi i campi 'username' e 'hostname'\n\n"
#define INSTALL_KEY_ERROR  "\nnon riesco a installare la chiave sull'host indicato\n\n"
#define EXE_COMMAND_ERROR "\ndevi riempire tutti i campi\n\n"
#define NO_SSH_DIR_ERROR "\nnon posso creare la directory ~/.ssh\n\n"
#define NO_SSH_CREATE_ERROR "\nnon sono riuscito a generare le chiavi\n\n"
#define NO_ENV_HOME_ERROR "\nviariabile di ambiente $HOME non definita\n\n"

/**************************************************/

#define get_entry_text(entry_name) gtk_entry_get_text (GTK_ENTRY (lookup_widget (GTK_WIDGET (gtksshlogin), entry_name)))
#define set_entry_text(entry_name, testo) gtk_entry_set_text (GTK_ENTRY (lookup_widget (GTK_WIDGET (gtksshlogin), entry_name)),testo)
#define get_whats_key(event) (event->keyval)
#define safe_free(p) { if(p){free(p); p = NULL;} }


void
cripta_password (const char *entry_name)
{
  static int noloop = 0;
	
  GtkWidget *entry;
  char *string = NULL;
  int i;

  if(noloop) return;
  noloop = 1;
	
  if( (entry  = lookup_widget (GTK_WIDGET (gtksshlogin), entry_name)) != NULL
  &&  (string = strdup(gtk_entry_get_text (GTK_ENTRY (entry))))  != NULL){ 
	g_print("\npassword is '%s'\n", string);  
	i = 0; while(string[i]) string[i++] = '*';
	gtk_entry_set_text (GTK_ENTRY (entry), string);
	safe_free(string); 
  } else {
	g_print("\nwidget '%s' doesn't exist!\n", entry_name);
  }
  
  noloop = 0;
} 


gboolean
on_entry_password_key_press_event      (GtkWidget       *widget,
                                        GdkEventKey     *event,
                                        gpointer         user_data)
{
  static int i = 0;
  int key = get_whats_key (event);
  g_print ("\nKey 0x%x pressed on password box\n", key);
	
  switch (key) {
	case KEYB_INVIO:
		break;
	
	case KEYB_BACKSPACE:
		if(i >= 0) {
			password[--i] = 0;
		}
		break;
		
	default:
		if(key >= ' ' && key <= 'z' && i < MAX_PASSWORD_LEN) {
			password[i++] = key;
			password[i] = 0;
		}
		break;
  }
  g_print ("\npassword = '%s'\n", password);
  
  return FALSE;
}

void
on_entry_password_changed              (GtkEditable     *editable,
                                        gpointer         user_data)
{
	cripta_password("entry_password");
}


/*******************************************************************/


void
show_message(const char *titolo, const char *testo)
{
  if(!message) message = create_message();
  gtk_label_set (GTK_LABEL(lookup_widget(message, "label_testo")), testo);
  gtk_label_set (GTK_LABEL(lookup_widget(message, "label_messaggio")), titolo);
  gtk_widget_show(message);
}

#define show_error_message(msg) show_message("ERRRORE", (msg))
#define show_info_message(msg)  show_message("INFORMAZIONE", (msg))

int
genera_chiavi(void) 
{
	int ret = 1;
	FILE *fp;
	
	if(!flag_di_campo[CM_PASSPHRASE] || !flag_di_campo[CM_ASKPASS]) {
		g_print("\nnon uso askpass\n");
		fp = popen("SSH_ASKPASS=""; export SSH_ASKPASS; mkdir -p ~/.ssh/old 2>/dev/null; mv -f ~/.ssh/id_dsa* ~/.ssh/old 2>/dev/null; ssh-keygen -t dsa\n", "w"); 
	} else {
		fp = popen("mkdir -p ~/.ssh/old 2>/dev/null; mv -f ~/.ssh/id_dsa* ~/.ssh/old 2>/dev/null; ssh-keygen -t dsa\n", "w"); 
	}
	
	if(fp) {
		fprintf(fp,"\n\n\n");
		fclose(fp);
		ret = 0;
	}

	if(!flag_di_campo[CM_PASSPHRASE])
		setenv("SSH_ASKPASS", stringa_di_campo[CM_ASKPASS], 1);
	
	return ret;
}

void
gtk_update(void)
{
	while (gtk_events_pending())
		gtk_main_iteration();
}


int
esegui_comando(void) 
{
	int ret;
	char str[1024];
	char cmd[1024];

	if( !strchr(stringa_di_campo[CM_USERNAME], '@') )
		sprintf(cmd,"ssh %s@%s \\\"%s\\\"\"", 
			stringa_di_campo[CM_USERNAME],
			stringa_di_campo[CM_HOSTNAME],
			stringa_di_campo[CM_COMMAND]);
	else
		sprintf(cmd,"ssh %s \\\"%s\\\"\"", 
			stringa_di_campo[CM_USERNAME],
			stringa_di_campo[CM_COMMAND]);
		
	g_print("\n%s\n", cmd);
	//xvt -sb +rv -T "" -e ""
	sprintf(str,"xterm -sb -hold -T \"gtksshlogin X-terminal\" -e \"resize -s 40 80;%s\n", cmd);
	
	gtk_widget_hide(gtksshlogin);
	gtk_update();
	
	FILE *fp = popen(str, "r"); 
	if(fp) {
		fclose(fp);
		ret = 0;
	} else {
		ret = 1;
	}
	
	MOSTRA_AL_CENTRO(gtksshlogin);
	return ret;
}


int
check_user_host_installed_keys(void)
{
	int ret = TRUE;
	
	if (!stringa_di_campo[CM_HOSTNAME] || !stringa_di_campo[CM_USERNAME])
		return FALSE;
	
	if(stringa_di_campo[CM_HOSTNAME][0]) {
		static char str[256];
		sprintf(str,"%s@%s", stringa_di_campo[CM_USERNAME], stringa_di_campo[CM_HOSTNAME]);
		if(!user_host_installed_keys_list_compare(str))
			ret = FALSE;
	} else {
		if(!user_host_installed_keys_list_compare(stringa_di_campo[CM_USERNAME]))
			ret = FALSE;
	}
	return ret;
}


/**************************************************/






void
on_button_cancella_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{
	//flag_di_campo[CM_USERNAME] = 0;
	//stringa_di_campo[CM_USERNAME] = NULL;
	set_entry_text("entry_username", "");
	
	//flag_di_campo[CM_HOSTNAME] = 0;
	//stringa_di_campo[CM_HOSTNAME] = NULL;
	set_entry_text("entry_hostname", "");
	
	//flag_di_campo[CM_COMMAND] = 0;
	//stringa_di_campo[CM_COMMAND]  = NULL;
	set_entry_text("entry_command", "");
	
	//gtk_widget_set_sensitive (lookup_widget(gtksshlogin, "button_installa"), FALSE);
	//gtk_widget_set_sensitive (lookup_widget(gtksshlogin, "button_esegui"), FALSE);
}




void
on_button_esegui_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
	/* Sono solo puntatori */
	//stringa_di_campo[CM_PASSWORD] = password;
	//stringa_di_campo[CM_USERNAME] = get_entry_text("entry_username");
	//stringa_di_campo[CM_HOSTNAME] = get_entry_text("entry_hostname");
	//stringa_di_campo[CM_COMMAND]  = get_entry_text("entry_command");
	/* se ci sono degli " nel comando occorre trasformarli in \" */
	esegui_comando();
	
	
	/*
	if(flag_di_campo[CM_USERNAME] 
	&& flag_di_campo[CM_HOSTNAME] 
	&& flag_di_campo[CM_COMMAND]) {
		esegui_comando();
	} else {
		show_message(EXE_COMMAND_ERROR);
	}
	*/
}


void
on_entry_username_changed              (GtkEditable     *editable,
                                        gpointer         user_data)
{
	int noat, state;
	
	//prima in assoluto
	stringa_di_campo[CM_USERNAME] = get_entry_text("entry_username");
	   flag_di_campo[CM_USERNAME] = strlen(stringa_di_campo[CM_USERNAME]);
	
	noat = !strchr(stringa_di_campo[CM_USERNAME], '@');
	gtk_widget_set_sensitive (lookup_widget(gtksshlogin, "entry_hostname"), noat);
	gtk_widget_set_sensitive (lookup_widget(gtksshlogin, "label_hostname"), noat);

	if(!noat) 
		set_entry_text("entry_hostname", "");
	
	state = flag_di_campo[CM_USERNAME] 
	     &&(flag_di_campo[CM_HOSTNAME] || !noat) 
	     && check_user_host_installed_keys();
	gtk_widget_set_sensitive (lookup_widget(gtksshlogin, "button_installa"), state);
	
	state = flag_di_campo[CM_COMMAND] 
	     && flag_di_campo[CM_USERNAME] 
	     &&(flag_di_campo[CM_HOSTNAME] || !noat);
	gtk_widget_set_sensitive (lookup_widget(gtksshlogin, "button_esegui"), state);
}


void
on_entry_hostname_changed              (GtkEditable     *editable,
                                        gpointer         user_data)
{
	int noat,state;
	
	//prima riga in assoluto
	stringa_di_campo[CM_HOSTNAME] = get_entry_text("entry_hostname");
	   flag_di_campo[CM_HOSTNAME] = strlen(stringa_di_campo[CM_HOSTNAME]);
	
	noat = !strchr(stringa_di_campo[CM_USERNAME], '@');
	
	state = flag_di_campo[CM_USERNAME] 
	     &&(flag_di_campo[CM_HOSTNAME] || !noat) 
	     && check_user_host_installed_keys();
	gtk_widget_set_sensitive (lookup_widget(gtksshlogin, "button_installa"), state);

	state = (flag_di_campo[CM_COMMAND] || flag_di_campo[CM_SESSION])
	     && flag_di_campo[CM_USERNAME] 
	     &&(flag_di_campo[CM_HOSTNAME] || !noat);
	gtk_widget_set_sensitive (lookup_widget(gtksshlogin, "button_esegui"), state);
}


void
on_entry_command_changed               (GtkEditable     *editable,
                                        gpointer         user_data)
{
	int state;
	
	//prima riga in assoluto
	stringa_di_campo[CM_COMMAND] = get_entry_text("entry_command");
	   flag_di_campo[CM_COMMAND] = strlen(stringa_di_campo[CM_COMMAND]);
	
	state =(flag_di_campo[CM_COMMAND] || flag_di_campo[CM_SESSION])
	     && flag_di_campo[CM_USERNAME] 
	     &&(flag_di_campo[CM_HOSTNAME] || !!strchr(stringa_di_campo[CM_USERNAME], '@'));
	gtk_widget_set_sensitive (lookup_widget(gtksshlogin, "button_esegui"), state);
}




void
on_button_chiudi_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
	gtk_widget_hide(message);
}


void
on_message_destroy                     (GtkObject       *object,
                                        gpointer         user_data)
{
	message = NULL;
}



void
on_gtksshlogin_destroy                      (GtkObject       *object,
                                        gpointer         user_data)
{
	user_host_installed_keys_list_savedown();
	g_print("\nRoberto Foglietta <robang@libero.it>\n");
	gtk_exit(0);
}



void
on_button_installa_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{
	char cmdstr[1024];
	
	gtk_widget_hide(gtksshlogin);
	gtk_update();
	
	sprintf(cmdstr,"xterm -sb -hold -T \"gtksshlogin X-terminal\" -e \"resize -s 4 80 2>&1 >/dev/null;scp %s/.ssh/id_dsa.pub %s@%s:~/.ssh/authorized_keys2; if [ $? = 0 ]; then echo \\\"INSTALLAZIONE CHIAVE RIUSCITA!\\\"; fi\"\n",
		getenv("HOME"),
		stringa_di_campo[CM_USERNAME],
		stringa_di_campo[CM_HOSTNAME]);
		
	g_print("\n%s\n",cmdstr);
	FILE *fp = popen(cmdstr, "w");
	if(fp) { 
		fclose(fp);
		
		sprintf(cmdstr,"%s@%s", stringa_di_campo[CM_USERNAME], 
					stringa_di_campo[CM_HOSTNAME]);
		if(!user_host_installed_keys_list_append(cmdstr)) {
			user_host_installed_keys_list_update();
			set_entry_text("entry_username", cmdstr);
		}
	} else 
		show_error_message(INSTALL_KEY_ERROR);
		
	MOSTRA_AL_CENTRO(gtksshlogin);
}




void
on_button_genera_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
	struct stat buf;
	int dir;
	char *home, path[256];

	user_host_installed_keys_list_resetall();
	
	home = getenv("HOME");
	if(home) {
		sprintf(path,"%s/.ssh", home);
		dir = stat(path, &buf);
		if(S_ISDIR(buf.st_mode) != 1) {
			mkdir(path,(mode_t)16877);
			dir = stat(path, &buf);
			if(S_ISDIR(buf.st_mode) != 1) {
				show_error_message(NO_SSH_DIR_ERROR);
			}
		}
		g_print("\n stat %s returned %d [%d] %d\n", home, dir, S_ISDIR(buf.st_mode), buf.st_mode);
		
		sprintf(path,"%s/id_dsa", path);
		int err = access(path, R_OK | F_OK);
		sprintf(path,"%s/.pub", path);
		err += access(path, R_OK | F_OK);
		if(err) err = genera_chiavi();
		if(err) {
			show_error_message(NO_SSH_CREATE_ERROR);
		} else {
			gtk_widget_hide(lookup_widget(gtksshlogin, "label_genera"));
			gtk_widget_hide(lookup_widget(gtksshlogin, "button_genera"));	
			gtk_widget_set_sensitive (lookup_widget(gtksshlogin, "checkbutton_passphrase"), FALSE);
			gtk_widget_set_sensitive (lookup_widget(gtksshlogin, "label_passphrase"), FALSE);
		}
	} else
		show_error_message(NO_ENV_HOME_ERROR);
	
	
	if(check_keys()) {
		gtk_widget_show(lookup_widget(gtksshlogin, "label_genera"));
		gtk_widget_show(lookup_widget(gtksshlogin, "button_genera"));
		if(flag_di_campo[CM_ASKPASS]) {
			g_print("\nattivo l'uso delle passphrase\n");
			gtk_widget_set_sensitive (lookup_widget(gtksshlogin, "checkbutton_passphrase"), TRUE);
			gtk_widget_set_sensitive (lookup_widget(gtksshlogin, "label_passphrase"), TRUE);
		}
	} else {
		gtk_widget_show(lookup_widget(gtksshlogin, "label_installa"));
		gtk_widget_show(lookup_widget(gtksshlogin, "button_installa"));
	}
}


void
on_checkbutton_passphrase_toggled      (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	flag_di_campo[CM_PASSPHRASE] = gtk_toggle_button_get_active (togglebutton);	
}


void
on_checkbutton_session_toggled         (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	flag_di_campo[CM_SESSION] = gtk_toggle_button_get_active (togglebutton);	
	g_print("\nsessione interativa: %s\n", (flag_di_campo[CM_SESSION])?"ON":"OFF");

	set_entry_text("entry_command", "");
	gtk_widget_set_sensitive (lookup_widget(gtksshlogin, "entry_command"), !flag_di_campo[CM_SESSION]);
	gtk_widget_set_sensitive (lookup_widget(gtksshlogin, "label_command"), !flag_di_campo[CM_SESSION]);
}

void
on_help_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	on_about_activate(menuitem, user_data);
}


void
on_about_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	int i;
	static char aboutmsg[1024]={""};
	
	if(!aboutmsg[0])
		for(i = 0; i < INFO_TESTO_MAX_NUM; i++)
			strcat(aboutmsg,infotesto[i]);
		
	show_info_message(aboutmsg);
}


void
on_esci_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	gtk_widget_destroy(gtksshlogin);
}


void
on_installa_chiave_activate            (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_scelta_terminale_activate           (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}



void
on_aggiungi_tutti_activate             (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	flag_di_campo[CM_ADDUSER] = ADDUSER_ALL;
}


void
on_aggiungi_chiavi_activate            (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	flag_di_campo[CM_ADDUSER] = ADDUSER_KEY;
}


void
on_aggiungi_nessuno_activate           (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	flag_di_campo[CM_ADDUSER] = ADDUSER_NO1;
}


void
on_aggiungi_amano_activate             (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}




void
on_nascondi_elenco_utenti_activate     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	if(get_menuitem_status(menuitem))
		user_host_installed_keys_list_hide();
	else
		user_host_installed_keys_list_show();
}


void
on_elimina_elenco_utenti_activate      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	g_print("\nelenco utenti eliminati\n");
	user_host_installed_keys_list_resetall();
	user_host_installed_keys_list_update();
}


void
on_aggiungi_chiave_activate            (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_sovrascrivi_chiavi_activate         (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_rigenera_chiavi_activate            (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_reinstalla_chiave_activate          (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_xterm_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_xvt_activate                        (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_nero_su_bianco_activate             (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_biano_su_nero_activate              (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_80colon_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_132colon_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_24righe_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_25righe_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_30righe_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_40righe_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_60righe_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_nessuna_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_64hist_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_128his_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_256his_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_25x80_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_sessione_interattiva_activate       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_altre_opzioni_activate              (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}

